"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useLocalForm = exports.useForm = void 0;
var react_1 = __importDefault(require("react"));
var hooks_1 = require("./hooks");
var utils_1 = require("./utils");
var validators_1 = require("./validators");
/**
 * Cases: Required (localvalidation), optional, async validation, custom validation messages
 * local state, global state
 * adaptor for custom form items
 * type check everything after "name"
 *
 * Must implement:
 * - Data type passed in, initial data
 * - create a form item
 * - onBlur for focus then unfocus for "required"
 *
 * Does not support onChange with multiple params
 */
var useForm = function (_a) {
    var values = _a.values, onChange = _a.onChange;
    var getIsMounted = hooks_1.useIsMounted();
    var _b = react_1.default.useState({}), touched = _b[0], setTouched = _b[1];
    var _c = react_1.default.useState({}), errors = _c[0], setErrors = _c[1];
    var _d = react_1.default.useState({}), loading = _d[0], setLoading = _d[1];
    var _e = react_1.default.useState({}), lastSeen = _e[0], setLastSeen = _e[1];
    var lastSeenRef = react_1.default.useRef({});
    var createFormItem = function (key, _a) {
        if (_a === void 0) { _a = {}; }
        var adaptor = _a.adaptor, meta = _a.meta, validationParams = __rest(_a, ["adaptor", "meta"]);
        return function (formItem) {
            var _a;
            handleErrorText({
                key: key,
                values: values,
                meta: meta,
                validationParams: validationParams,
                lastSeen: lastSeen,
                setLastSeen: setLastSeen,
                lastSeenRef: lastSeenRef,
                loading: loading,
                setLoading: setLoading,
                errors: errors,
                setErrors: setErrors,
                getIsMounted: getIsMounted,
            });
            var onChangeValue = function (value) {
                var _a, _b;
                // https://stackoverflow.com/questions/60456679/assignment-of-generic-object-to-partial-type
                onChange((_a = {}, _a[key] = value, _a));
                setTouched(__assign(__assign({}, touched), (_b = {}, _b[key] = true, _b)));
            };
            var onChangeHandler = adaptor
                ? function (v) { return onChangeValue(adaptor(v)); }
                : onChange;
            var formItemProps = __assign({ name: key, value: values[key], onChange: onChangeHandler, onBlur: onChangeHandler, errorText: (_a = (touched[key] && errors[key])) !== null && _a !== void 0 ? _a : null, meta: meta }, (validationParams.customAsync
                ? {
                    isLoading: !!loading[key],
                }
                : {}));
            return formItem(formItemProps);
        };
    };
    var validate = function () {
        var allTouched = utils_1.ObjectKeys(values).reduce(function (a, n) {
            a[n] = true;
            return a;
        }, {});
        setTouched(allTouched);
        var numErrors = Object.values(errors).filter(function (value) { return typeof value === "string"; }).length;
        var numLoading = Object.values(loading).filter(function (value) { return value; }).length;
        return !numErrors && !numLoading;
    };
    return {
        createFormItem: createFormItem,
        validate: validate,
    };
};
exports.useForm = useForm;
var getErrorTextFn = function (_a) {
    var props = _a.props, required = _a.required, validationMessages = _a.validationMessages, custom = _a.custom, customAsync = _a.customAsync, typeParams = __rest(_a, ["props", "required", "validationMessages", "custom", "customAsync"]);
    return function (value) {
        var _a, _b;
        if (required && !value) {
            return (_b = (_a = validationMessages === null || validationMessages === void 0 ? void 0 : validationMessages.required) === null || _a === void 0 ? void 0 : _a.call(validationMessages, props)) !== null && _b !== void 0 ? _b : "field is required";
        }
        else {
            if (typeof value === "string" && typeParams.validation) {
                var result = validators_1.validator(typeParams.validation, value);
                if (result)
                    return result;
            }
            if (custom) {
                var result = custom(value);
                if (result)
                    return result;
            }
            if (customAsync)
                return customAsync(value);
        }
        return null;
    };
};
var handleErrorText = function (_a) {
    var _b, _c, _d, _e, _f;
    var key = _a.key, values = _a.values, meta = _a.meta, validationParams = _a.validationParams, lastSeen = _a.lastSeen, setLastSeen = _a.setLastSeen, lastSeenRef = _a.lastSeenRef, loading = _a.loading, setLoading = _a.setLoading, errors = _a.errors, setErrors = _a.setErrors, getIsMounted = _a.getIsMounted;
    var getErrorText = getErrorTextFn(__assign({ props: {
            name: key,
            value: values[key],
            meta: meta,
        } }, validationParams));
    if (lastSeen[key] !== values[key]) {
        setLastSeen(__assign(__assign({}, lastSeen), (_b = {}, _b[key] = values[key], _b)));
        lastSeenRef.current = __assign(__assign({}, lastSeenRef.current), (_c = {}, _c[key] = values[key], _c));
        var errorText = getErrorText(values[key]);
        if (errorText instanceof Promise) {
            setLoading(__assign(__assign({}, loading), (_d = {}, _d[key] = true, _d)));
            setErrors(__assign(__assign({}, errors), (_e = {}, _e[key] = null, _e)));
            errorText
                .then(function (errorText) {
                var _a;
                return getIsMounted() &&
                    values[key] === lastSeenRef.current[key] &&
                    setErrors(__assign(__assign({}, errors), (_a = {}, _a[key] = errorText, _a)));
            })
                .catch(function (err) {
                var _a;
                return getIsMounted() &&
                    err instanceof Error &&
                    setErrors(__assign(__assign({}, errors), (_a = {}, _a[key] = err.message, _a)));
            })
                .then(function () {
                var _a;
                setLoading(__assign(__assign({}, loading), (_a = {}, _a[key] = false, _a)));
            });
        }
        else if (errorText !== errors[key]) {
            setErrors(__assign(__assign({}, errors), (_f = {}, _f[key] = errorText, _f)));
        }
    }
};
var useLocalForm = function (_a) {
    var initialData = _a.initialData;
    var _b = react_1.default.useState(initialData), formState = _b[0], setFormState = _b[1];
    return __assign(__assign({}, exports.useForm({
        values: formState,
        onChange: function (val) { return setFormState(__assign(__assign({}, formState), val)); },
    })), { getValues: function () { return formState; } });
};
exports.useLocalForm = useLocalForm;
